// Target Color
const getContrastingTextColor = rgb => vectorDot(rgb, [0.3, 0.6, 0.1]) >= 128 ? "#222" : "#ddd";

const onRandomColor = state => {
  const color = rgb2hex([Math.random(), Math.random(), Math.random()].map(c => c * 255));
  document.getElementById("color-input").value = color;
  onColorChanged(state, color); // triggers rescore
};

const onColorChanged = (state, newValue) => {
  const colorInput = "#" + (newValue?.replace("#", "") ?? "FFFFFF");

  if (colorInput.length !== 7) {
    return;
  }

  const rgb = d3.color(colorInput);
  if (!rgb) {
    return;
  }
  const { J, a, b } = d3.jab(rgb);

  state.target = {
    jab: buildVectorData([J, a, b], jab2hue, jab2hex),
    rgb: buildVectorData([rgb.r, rgb.g, rgb.b], rgb2hue, rgb2hex),
  };

  const rootElem = document.querySelector(":root");
  rootElem.style.setProperty("--background", state.target.rgb.hex);
  rootElem.style.setProperty("--highlight", getContrastingTextColor(state.target.rgb.vector));

  rescoreAll(state.target);
  onControlsChanged(state);
};

// Sorting/Metrics
const scaleOptions = {
  none: () => [1, 1, 1],
  direct: data => data.clusters.map(c => c.size / data.total.size),
  inverse: data => data.clusters.map(c => data.total.size / c.size),
};

const scaleOptionsDisplay = {
  none: () => ["", ""],
  direct: arg => [`\\frac{\\left|${arg}\\right|}{\\left|P\\right|} \\left(`, "\\right)"],
  inverse: arg => [`\\frac{\\left|P\\right|}{\\left|${arg}\\right|} \\left(`, "\\right)"],
}

const sortOrders = {
  max: (a, b) => b - a,
  min: (a, b) => a - b,
};

const onControlsChanged = state => {
  // Update metric displays
  document.getElementById("metric-display").innerHTML = TeXZilla.toMathMLString(`
    ${metrics[state.sortMetric].displayName("P")} = ${metrics[state.sortMetric].displayBody("P")}
  `);
  let arg;
  if (state.useCluster) {
    arg = "K\\left(P\\right)";
    const [ clusterScaleDisplayL, clusterScaleDisplayR ] = scaleOptionsDisplay[state.clusterSettings.scaleOption]("P_i");
    document.getElementById("cluster-metric-display").innerHTML = TeXZilla.toMathMLString(`
      K\\left(P\\right) = \\arg\\${state.clusterSettings.sortOrder}_{P_i}\\left[
        ${clusterScaleDisplayL} ${metrics[state.clusterSettings.sortMetric].displayBody("P_i")} ${clusterScaleDisplayR}
      \\right]
    `);
  } else {
    arg = "P";
    document.getElementById("cluster-metric-display").innerHTML = '';
  }
  const [ scaleDisplayL, scaleDisplayR ] = state.useCluster ? scaleOptionsDisplay[state.scaleOption](arg) : ["", ""];
  document.getElementById("final-metric-display").innerHTML = TeXZilla.toMathMLString(`
    \\begin{aligned}
    &\\${state.sortOrder}_{P}\\left[
      ${scaleDisplayL.slice(0, -6)} ${metrics[state.sortMetric].displayName(arg)}
    \\right]\\\\
    = 
    &\\${state.sortOrder}_{P}\\left[
      ${scaleDisplayL} ${metrics[state.sortMetric].displayBody(arg)} ${scaleDisplayR}
    \\right]
    \\end{aligned}
  `);

  // Update the current best
  getBest(
    state.number, state.space, 
    state.useCluster ? { sortMetric: state.clusterSettings.sortMetric, sortOrder: sortOrders[state.clusterSettings.sortOrder], scaleOption: scaleOptions[state.clusterSettings.scaleOption] } : null, 
    { sortMetric: state.sortMetric, sortOrder: sortOrders[state.sortOrder], scaleOption: scaleOptions[state.scaleOption] }
  ).forEach(pkmn => console.log(pkmn));
  // TODO actually render to output list instead
};