const metrics = {
  sigma: { // RMS
    displayName: p => String.raw`\sigma\left(${p}\right)`,
    displayBody: p => String.raw`
      \sqrt{I\left(${p}\right) - 2\vec{q}\cdot\vec{\mu}\left(${p}\right) + \left|\left|\vec{q}\right|\right|^2}
    `,
    evaluate: (data, target) => Math.sqrt(data.inertia - 2 * vectorDot(data.mu.vector, target.vector) + target.sqMag),
  },
  bigTheta: { // mean of angle
    displayName: p => String.raw`\Theta\left(${p}\right)`,
    displayBody: p => String.raw`
      \cos^{-1}\left( \hat{q}\cdot\vec{\nu}\left(${p}\right) \right)
    `,
    evaluate: (data, target) => rad2deg * Math.acos(vectorDot(data.nu, target.unit)),
  },
  theta: { // angle of mean
    displayName: p => String.raw`\theta\left(${p}\right)`,
    displayBody: p => String.raw`
      \cos^{-1}\left( \hat{q}\cdot\hat{\mu}\left(${p}\right) \right)
    `,
    evaluate: (data, target) => rad2deg * Math.acos(vectorDot(data.mu.unit, target.unit)),
  },
  phi: { // hue angle
    displayName: p => String.raw`\phi\left(${p}\right)`,
    displayBody: p => String.raw`
      \angle \left(\vec{q}_{\perp}, \vec{\mu}\left(${p}\right)_{\perp} \right)
    `,
    evaluate: (data, target) => angleDiff(data.mu.hue, target.hue),
  },
  delta: { // euclidean
    displayName: p => String.raw`\delta\left(${p}\right)`,
    displayBody: p => String.raw`
      \left|\left| \vec{q} - \vec{\mu}\left(${p}\right) \right|\right|
    `,
    evaluate: (data, target) => vectorDist(data.mu.vector, target.vector),
  },
  ch: { // chebyshev
    displayName: p => String.raw`Ч\left(${p}\right)`,
    displayBody: p => String.raw`
      \max_{i} \left| \vec{\mu}\left(${p}\right)_i - \vec{q}_i \right|
    `,
    evaluate: (data, target) => Math.max(...data.mu.vector.map((x, i) => Math.abs(x - target.vector[i]))),
  },
  inertia: {
    displayName: p => String.raw`I\left(${p}\right)`,
    displayBody: p => String.raw`
      \frac{1}{\left|${p}\right|} \sum_{p\in ${p}}{\left|\left|\vec{p}\right|\right|^2}
    `,
    evaluate: data => data.inertia,
  },
  size: {
    displayName: p => String.raw`N\left(${p}\right)`,
    displayBody: p => String.raw`\left|${p}\right|`,
    evaluate: data => data.size,
  }
};

const applyMetrics = (data, target) => Object.fromEntries(
  Object.entries(metrics).map(([name, metric]) => [name, metric.evaluate(data, target)])
);
